const { MongoClient } = require('mongodb');
const { handleRestfulGetRequest, handleRestfulPostRequest, handleRestfulPutRequest, handleRestfulPatchRequest, handleRestfulDeleteRequest } = require('./rest_handlers');
const { handleUserRequests } = require('./user_handlers');

let db;
const defaultOptions = { pageSize: 20, handleUsers: true };

const connectToDatabase = async (req) => {
  if (!db) {
    const dbName = process.env.DB_NAME;
    const dbUri = process.env.DB_URI;
    const client = await MongoClient.connect(dbUri, { useNewUrlParser: true });
    db = client.db(dbName);
  }
  req.db = db;
};

const parseRequest = async (req) => {
  const method = req.method.toUpperCase();
  const urlSplit = req.path.split('/');
  urlSplit.shift(); // remove the leading forward slash
  const resource = urlSplit.shift().toLowerCase();
  const id = urlSplit.shift();
  req.method = method;
  req.resource = resource;
  req.id = id;
};

module.exports = (options) => {
  const opts = Object.assign(defaultOptions, options);
  return async (req, res, next) => {
    try {
      req.opts = opts;
      await connectToDatabase(req);
      await parseRequest(req);
      if (opts.handleUsers) {
        await handleUserRequests(req, res, next);
      }
      await handleRestfulGetRequest(req, res);
      await handleRestfulPostRequest(req, res);
      await handleRestfulPutRequest(req, res);
      await handleRestfulPatchRequest(req, res);
      await handleRestfulDeleteRequest(req, res);
      if (!res.headersSent) {
        next();
      }
    } catch (err) {
      next(err);
    }
  };

};
